import { EditorView } from '@codemirror/view';
import { HighlightStyle, tags } from '@codemirror/highlight';

const config = {
    name: 'cobalt',
    dark: true,
    background: '#0022BB',
    foreground: '#FFFFFF',
    selection: '#9933CC',
    cursor: '#F0F000',
    dropdownBackground: '#0022BB',
    dropdownBorder: '#6699CC',
    activeLine: '#0011AA',
    matchingBracket: '#0011AA',
    keyword: '#B0B0B0',
    storage: '#C0E0F0',
    variable: '',
    parameter: '#B0D0F0',
    function: '#D0B0F0',
    string: '#B0F0D0',
    constant: '#F0D0E0',
    type: '#E0C0F0',
    class: '#F0D0B0',
    number: '#B0D0F0',
    comment: '#6090F0',
    heading: '#ddbb88',
    invalid: '#F8F8F0',
    regexp: '#B0F0D0',
};
const cobaltTheme = EditorView.theme({
    '&': {
        color: config.foreground,
        backgroundColor: config.background,
    },
    '.cm-content': { caretColor: config.cursor },
    '&.cm-focused .cm-cursor': { borderLeftColor: config.cursor },
    '&.cm-focused .cm-selectionBackground, .cm-selectionBackground, & ::selection': { backgroundColor: config.selection },
    '.cm-panels': { backgroundColor: config.dropdownBackground, color: config.foreground },
    '.cm-panels.cm-panels-top': { borderBottom: '2px solid black' },
    '.cm-panels.cm-panels-bottom': { borderTop: '2px solid black' },
    '.cm-searchMatch': {
        backgroundColor: config.dropdownBackground,
        outline: `1px solid ${config.dropdownBorder}`
    },
    '.cm-searchMatch.cm-searchMatch-selected': {
        backgroundColor: config.selection
    },
    '.cm-activeLine': { backgroundColor: config.activeLine },
    '.cm-activeLineGutter': { backgroundColor: config.background },
    '.cm-selectionMatch': { backgroundColor: config.selection },
    '.cm-matchingBracket, .cm-nonmatchingBracket': {
        backgroundColor: config.matchingBracket,
        outline: 'none'
    },
    '.cm-gutters': {
        backgroundColor: config.background,
        color: config.foreground,
        border: 'none'
    },
    '.cm-lineNumbers, .cm-gutterElement': { color: 'inherit' },
    '.cm-foldPlaceholder': {
        backgroundColor: 'transparent',
        border: 'none',
        color: config.foreground
    },
    '.cm-tooltip': {
        border: `1px solid ${config.dropdownBorder}`,
        backgroundColor: config.dropdownBackground,
        color: config.foreground
    },
    '.cm-tooltip.cm-tooltip-autocomplete': {
        '& > ul > li[aria-selected]': {
            background: config.selection,
            color: config.foreground
        }
    },
}, { dark: config.dark });
const cobaltHighlightStyle = HighlightStyle.define([
    // const, let, function, if
    { tag: tags.keyword, color: config.keyword },
    // document
    { tag: [tags.name, tags.deleted, tags.character, tags.macroName], color: config.variable },
    // getElementById
    { tag: [tags.propertyName], color: config.function },
    // "string"
    { tag: [tags.processingInstruction, tags.string, tags.inserted, tags.special(tags.string)], color: config.string },
    // render
    { tag: [tags.function(tags.variableName), tags.labelName], color: config.function },
    // ???
    { tag: [tags.color, tags.constant(tags.name), tags.standard(tags.name)], color: config.constant },
    // btn, count, fn render()
    { tag: [tags.definition(tags.name), tags.separator], color: config.variable },
    { tag: [tags.className], color: config.class },
    { tag: [tags.number, tags.changed, tags.annotation, tags.modifier, tags.self, tags.namespace], color: config.number },
    { tag: [tags.typeName], color: config.type, fontStyle: config.type },
    { tag: [tags.operator, tags.operatorKeyword], color: config.keyword },
    { tag: [tags.url, tags.escape, tags.regexp, tags.link], color: config.regexp },
    { tag: [tags.meta, tags.comment], color: config.comment },
    { tag: tags.strong, fontWeight: 'bold' },
    { tag: tags.emphasis, fontStyle: 'italic' },
    { tag: tags.link, textDecoration: 'underline' },
    { tag: tags.heading, fontWeight: 'bold', color: config.heading },
    { tag: [tags.atom, tags.bool, tags.special(tags.variableName)], color: config.variable },
    { tag: tags.invalid, color: config.invalid },
    { tag: tags.strikethrough, textDecoration: 'line-through' },
]);
const cobalt = [
    cobaltTheme,
    cobaltHighlightStyle,
];

export { cobalt, cobaltHighlightStyle, cobaltTheme, config };
